/*
	ExternalInterface test code for GNU Flash Player
	Written (2016) by Nutchanon Wetchasit <Nutchanon.Wetchasit@gmail.com>
	
	Released to public domain under Creative Commons Zero 1.0
	(https://creativecommons.org/publicdomain/zero/1.0/)
*/
import flash.external.*;

class JS2FlashNoThisV2 {
	/** Entry point */
	public static function main(mc:MovieClip):Void {
		
		/** Variable-based trace function, log messages to `_root.trace_log` */
		var vtrace = function(msg):Void {
			trace(msg);
			if(_root.trace_log == undefined) _root.trace_log = msg + "\n";
			else _root.trace_log += (msg + "\n");
		};
		
		/** Checker functions, print each test result to trace() */
		var check = function(op, msg):Void {
			if(op) vtrace("PASSED: " + msg);
			else vtrace("FAILED: " + msg);
		};
		var check_equals = function(op1, op2, msg):Void {
			if(op1 == op2) vtrace("PASSED: " + msg);
			else vtrace("FAILED: " + msg + " (\"" + op1 + "\" != \"" + op2 + "\")");
		};
		
		check_equals(ExternalInterface.available, true, "ExternalInterface should be available");
		
		check_equals(
			ExternalInterface.addCallback("nullthis_call", null,
				function():Void {
					check(true, "nullthis_call ExternalInterface callback should be called");
					check_equals(this, undefined, "this pointer inside nullthis_call should be equal to undefined value");
					check_equals(this, null, "this pointer inside nullthis_call should be equal to null value");
					check(this !== undefined, "this pointer inside nullthis_call should not strictly equal to undefined value");
					check(this !== null, "this pointer inside nullthis_call should not strictly equal to null value");
					check_equals("" + this, "undefined", "Appending this pointer inside nullthis_call to empty string should give \"undefined\" string");
					check(this === _global, "this pointer inside nullthis_call should be strictly equal to _global object");
					this.nullthis_value = "nullthis_call";
				}
			), true, "nullthis_call ExternalInterface callback registration should not fail"
		);
		
		check_equals(
			ExternalInterface.addCallback("undefinedthis_call", undefined,
				function():Void {
					check(true, "undefinedthis_call ExternalInterface callback should be called");
					check_equals(this, undefined, "this pointer inside undefinedthis_call should be equal to undefined value");
					check_equals(this, null, "this pointer inside undefinedthis_call should be equal to null value");
					check(this !== undefined, "this pointer inside undefinedthis_call should not strictly equal to undefined value");
					check(this !== null, "this pointer inside undefinedthis_call should not strictly equal to null value");
					check_equals("" + this, "undefined", "Appending this pointer inside undefinedthis_call to empty string should give \"undefined\" string");
					check(this === _global, "this pointer inside undefinedthis_call should be strictly equal to _global object");
					this.undefinedthis_value = "undefinedthis_call";
				}
			), true, "undefinedthis_call ExternalInterface callback registration should not fail"
		);
		
		check_equals(
			ExternalInterface.addCallback("globalcheck_call", mc,
				function():Void {
					check(true, "globalcheck_call ExternalInterface callback should be called");
					check_equals(_global.nullthis_value, "nullthis_call", "_global.nullthis_value should be set to the correct value");
					check_equals(_global.undefinedthis_value, "undefinedthis_call", "_global.undefinedthis_value should be set to the correct value");
				}
			), true, "globalcheck_call ExternalInterface callback registration should not fail"
		);
	}
}
